@@ -1,22 +1,41 @@
+#
+# Copyright 2010 Ruediger Gad <r.c.g@gmx.de>
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+
+import cairo
 from datetime import datetime
 import gtk
 import gobject
 import hildon
 import hildondesktop
+import math
 
 class BinaryClockStatusAreaPlugin(hildondesktop.StatusMenuItem):
     def __init__(self):
         hildondesktop.StatusMenuItem.__init__(self)
 
         self.clockStyle = 0
-        self.showSeconds = True
+        self.showSeconds = False
 
         self.time = None
         self.oldTime = None
 
         button = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL, "Binary Clock")
         image = gtk.Image()
-        image.set_from_file("/usr/share/icons/hicolor/48x48/apps/binaryclock_status_area_plugin.png")
+        image.set_from_file("/usr/share/icons/hicolor/64x64/apps/binaryclock_status_area_plugin.png")
         button.set_image(image)
         button.set_alignment(0, 0.5, 0, 0)
         button.connect("clicked", self.settingsCb)
@@ -36,27 +55,38 @@ class BinaryClockStatusAreaPlugin(hildondesktop.StatusMenuItem):
         gobject.timeout_add(1000, self.updateCb)
 
     def drawBcdClock(self):
-        self.drawBitsVertically(2, 16, self.time.hour / 10, 2)
-        self.drawBitsVertically(12, 0, self.time.hour % 10, 4)
-        self.drawBitsVertically(26, 8, self.time.minute / 10, 3)
-        self.drawBitsVertically(36, 0, self.time.minute % 10, 4)
+        self.drawBitsVertically(2, 18, 3, self.time.hour / 10, 2)
+        self.drawBitsVertically(12, 0, 3, self.time.hour % 10, 4)
+        self.drawBitsVertically(26, 9, 3, self.time.minute / 10, 3)
+        self.drawBitsVertically(36, 0, 3, self.time.minute % 10, 4)
         if self.showSeconds:
-            self.drawBitsVertically(50, 8, self.time.second / 10, 3)
-            self.drawBitsVertically(60, 0, self.time.second % 10, 4)
+            self.drawBitsVertically(50, 9, 3, self.time.second / 10, 3)
+            self.drawBitsVertically(60, 0, 3, self.time.second % 10, 4)
 
     def drawBinaryClock(self):
-        self.drawBitsHorizontally(12, 2, self.time.hour, 5)
-        self.drawBitsHorizontally(0, 14, self.time.minute, 6)
         if self.showSeconds:
-            self.drawBitsHorizontally(0, 26, self.time.second, 6)
+            self.drawBitsHorizontally(12, 2, 4, self.time.hour, 5)
+            self.drawBitsHorizontally(0, 14, 4, self.time.minute, 6)
+            self.drawBitsHorizontally(0, 26, 4, self.time.second, 6)
+        else :
+            self.drawBitsHorizontally(18, 2, 6, self.time.hour, 5)
+            self.drawBitsHorizontally(0, 20, 6, self.time.minute, 6)
 
-    def drawBitsHorizontally(self, x, y, value, n):
+    def drawBitsHorizontally(self, x, y, radius, value, n):
         for i in range(n):
-            self.drawable.draw_arc(self.gc, (value >> (n - i - 1) & 1) > 0, x + 2 + (12 * i), y, 8, 8, 0, 360 *64)
+            self.context.arc(x + (1.5 * radius) + (3 * radius * i), y + radius, radius, 0, 2 * math.pi)
+            self.context.set_source_rgb(1, 1, 1)
+            if (value >> (n - i - 1) & 1) > 0:
+                self.context.fill_preserve()
+            self.context.stroke()
 
-    def drawBitsVertically(self, x, y, value, n):
+    def drawBitsVertically(self, x, y, radius, value, n):
         for i in range(n):
-            self.drawable.draw_arc(self.gc, (value >> (n - i - 1) & 1) > 0, x, y + 1 + (8 * i), 6, 6, 0, 360 *64)
+            self.context.arc(x + radius, y + (1.5 * radius)  + (3 * radius * i), radius, 0, 2 * math.pi)
+            self.context.set_source_rgb(1, 1, 1)
+            if (value >> (n - i - 1) & 1) > 0:
+                self.context.fill_preserve()
+            self.context.stroke()
 
     def drawClock(self):
         if self.clockStyle == 0:
@@ -67,9 +97,11 @@ class BinaryClockStatusAreaPlugin(hildondesktop.StatusMenuItem):
             self.drawBinaryClock()
 
     def exposeCb(self, widget, event):
-        self.drawable = self.drawingArea.window
-        self.gc = self.drawingArea.get_style().fg_gc[gtk.STATE_NORMAL]
-        self.gc.foreground = self.drawingArea.get_colormap().alloc_color(65535, 65535, 65535)
+        self.context = self.drawingArea.window.cairo_create()
+
+        self.context.set_source_rgba(1.0, 1.0, 1.0, 0.0)
+        self.context.set_operator(cairo.OPERATOR_SOURCE)
+        self.context.paint()
 
         self.drawClock()
         return False
@@ -77,6 +109,8 @@ class BinaryClockStatusAreaPlugin(hildondesktop.StatusMenuItem):
     def setSize(self):
         if (not self.showSeconds) and (self.clockStyle == 1):
             self.drawingArea.set_size_request(47, 36)
+        elif (not self.showSeconds) and (self.clockStyle == 0):
+            self.drawingArea.set_size_request(110, 36)
         else:
             self.drawingArea.set_size_request(76, 36)
 
